from enum import Enum

class WifiRadioType(str, Enum):
    Unknown = "unknown",
    Wifi80211a = "wifi80211a",
    Wifi80211b = "wifi80211b",
    Wifi80211g = "wifi80211g",
    Wifi80211n = "wifi80211n",
    Wifi80211ac = "wifi80211ac",
    Wifi80211ax = "wifi80211ax",
    UnknownFutureValue = "unknownFutureValue",

