from __future__ import annotations
import datetime
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from kiota_abstractions.store import BackedModel, BackingStore, BackingStoreFactorySingleton
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .attack_simulation_user import AttackSimulationUser

@dataclass
class AttackSimulationSimulationUserCoverage(AdditionalDataHolder, BackedModel, Parsable):
    # Stores model information.
    backing_store: BackingStore = field(default_factory=BackingStoreFactorySingleton(backing_store_factory=None).backing_store_factory.create_backing_store, repr=False)

    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)
    # User in an attack simulation and training campaign.
    attack_simulation_user: Optional[AttackSimulationUser] = None
    # Number of link clicks in the received payloads by the user in attack simulation and training campaigns.
    click_count: Optional[int] = None
    # Number of compromising actions by the user in attack simulation and training campaigns.
    compromised_count: Optional[int] = None
    # Date and time of the latest attack simulation and training campaign that the user was included in.
    latest_simulation_date_time: Optional[datetime.datetime] = None
    # The OdataType property
    odata_type: Optional[str] = None
    # Number of attack simulation and training campaigns that the user was included in.
    simulation_count: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: Optional[ParseNode] = None) -> AttackSimulationSimulationUserCoverage:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AttackSimulationSimulationUserCoverage
        """
        if not parse_node:
            raise TypeError("parse_node cannot be null.")
        return AttackSimulationSimulationUserCoverage()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .attack_simulation_user import AttackSimulationUser

        from .attack_simulation_user import AttackSimulationUser

        fields: Dict[str, Callable[[Any], None]] = {
            "attackSimulationUser": lambda n : setattr(self, 'attack_simulation_user', n.get_object_value(AttackSimulationUser)),
            "clickCount": lambda n : setattr(self, 'click_count', n.get_int_value()),
            "compromisedCount": lambda n : setattr(self, 'compromised_count', n.get_int_value()),
            "latestSimulationDateTime": lambda n : setattr(self, 'latest_simulation_date_time', n.get_datetime_value()),
            "@odata.type": lambda n : setattr(self, 'odata_type', n.get_str_value()),
            "simulationCount": lambda n : setattr(self, 'simulation_count', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if not writer:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("attackSimulationUser", self.attack_simulation_user)
        writer.write_int_value("clickCount", self.click_count)
        writer.write_int_value("compromisedCount", self.compromised_count)
        writer.write_datetime_value("latestSimulationDateTime", self.latest_simulation_date_time)
        writer.write_str_value("@odata.type", self.odata_type)
        writer.write_int_value("simulationCount", self.simulation_count)
        writer.write_additional_data_value(self.additional_data)
    

