from enum import Enum

class CalendarColor(str, Enum):
    Auto = "auto",
    LightBlue = "lightBlue",
    LightGreen = "lightGreen",
    LightOrange = "lightOrange",
    LightGray = "lightGray",
    LightYellow = "lightYellow",
    LightTeal = "lightTeal",
    LightPink = "lightPink",
    LightBrown = "lightBrown",
    LightRed = "lightRed",
    MaxColor = "maxColor",

