from enum import Enum

class WindowsMalwareExecutionState(str, Enum):
    # Unknown
    Unknown = "unknown",
    # Blocked
    Blocked = "blocked",
    # Allowed
    Allowed = "allowed",
    # Running
    Running = "running",
    # Not running
    NotRunning = "notRunning",

