from enum import Enum

class SynchronizationSecret(str, Enum):
    None_ = "None",
    UserName = "UserName",
    Password = "Password",
    SecretToken = "SecretToken",
    AppKey = "AppKey",
    BaseAddress = "BaseAddress",
    ClientIdentifier = "ClientIdentifier",
    ClientSecret = "ClientSecret",
    SingleSignOnType = "SingleSignOnType",
    Sandbox = "Sandbox",
    Url = "Url",
    Domain = "Domain",
    ConsumerKey = "ConsumerKey",
    ConsumerSecret = "ConsumerSecret",
    TokenKey = "TokenKey",
    TokenExpiration = "TokenExpiration",
    Oauth2AccessToken = "Oauth2AccessToken",
    Oauth2AccessTokenCreationTime = "Oauth2AccessTokenCreationTime",
    Oauth2RefreshToken = "Oauth2RefreshToken",
    SyncAll = "SyncAll",
    InstanceName = "InstanceName",
    Oauth2ClientId = "Oauth2ClientId",
    Oauth2ClientSecret = "Oauth2ClientSecret",
    CompanyId = "CompanyId",
    UpdateKeyOnSoftDelete = "UpdateKeyOnSoftDelete",
    SynchronizationSchedule = "SynchronizationSchedule",
    SystemOfRecord = "SystemOfRecord",
    SandboxName = "SandboxName",
    EnforceDomain = "EnforceDomain",
    SyncNotificationSettings = "SyncNotificationSettings",
    SkipOutOfScopeDeletions = "SkipOutOfScopeDeletions",
    Oauth2AuthorizationCode = "Oauth2AuthorizationCode",
    Oauth2RedirectUri = "Oauth2RedirectUri",
    ApplicationTemplateIdentifier = "ApplicationTemplateIdentifier",
    Oauth2TokenExchangeUri = "Oauth2TokenExchangeUri",
    Oauth2AuthorizationUri = "Oauth2AuthorizationUri",
    AuthenticationType = "AuthenticationType",
    Server = "Server",
    PerformInboundEntitlementGrants = "PerformInboundEntitlementGrants",
    HardDeletesEnabled = "HardDeletesEnabled",
    SyncAgentCompatibilityKey = "SyncAgentCompatibilityKey",
    SyncAgentADContainer = "SyncAgentADContainer",
    ValidateDomain = "ValidateDomain",
    TestReferences = "TestReferences",
    ConnectionString = "ConnectionString",

