from enum import Enum

class Windows10EditionType(str, Enum):
    # Windows 10 Enterprise
    Windows10Enterprise = "windows10Enterprise",
    # Windows 10 EnterpriseN
    Windows10EnterpriseN = "windows10EnterpriseN",
    # Windows 10 Education
    Windows10Education = "windows10Education",
    # Windows 10 EducationN
    Windows10EducationN = "windows10EducationN",
    # Windows 10 Mobile Enterprise
    Windows10MobileEnterprise = "windows10MobileEnterprise",
    # Windows 10 Holographic Enterprise
    Windows10HolographicEnterprise = "windows10HolographicEnterprise",
    # Windows 10 Professional
    Windows10Professional = "windows10Professional",
    # Windows 10 ProfessionalN
    Windows10ProfessionalN = "windows10ProfessionalN",
    # Windows 10 Professional Education
    Windows10ProfessionalEducation = "windows10ProfessionalEducation",
    # Windows 10 Professional EducationN
    Windows10ProfessionalEducationN = "windows10ProfessionalEducationN",
    # Windows 10 Professional for Workstations
    Windows10ProfessionalWorkstation = "windows10ProfessionalWorkstation",
    # Windows 10 Professional for Workstations N
    Windows10ProfessionalWorkstationN = "windows10ProfessionalWorkstationN",

