from __future__ import annotations
import datetime
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from kiota_abstractions.store import BackedModel, BackingStore, BackingStoreFactorySingleton
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .onenote_source_service import OnenoteSourceService
    from .recent_notebook_links import RecentNotebookLinks

@dataclass
class RecentNotebook(AdditionalDataHolder, BackedModel, Parsable):
    # Stores model information.
    backing_store: BackingStore = field(default_factory=BackingStoreFactorySingleton(backing_store_factory=None).backing_store_factory.create_backing_store, repr=False)

    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)
    # The name of the notebook.
    display_name: Optional[str] = None
    # The date and time when the notebook was last modified. The timestamp represents date and time information using ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 is 2014-01-01T00:00:00Z. Read-only.
    last_accessed_time: Optional[datetime.datetime] = None
    # Links for opening the notebook. The oneNoteClientURL link opens the notebook in the OneNote client, if it's installed. The oneNoteWebURL link opens the notebook in OneNote on the web.
    links: Optional[RecentNotebookLinks] = None
    # The OdataType property
    odata_type: Optional[str] = None
    # The backend store where the Notebook resides, either OneDriveForBusiness or OneDrive.
    source_service: Optional[OnenoteSourceService] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: Optional[ParseNode] = None) -> RecentNotebook:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RecentNotebook
        """
        if not parse_node:
            raise TypeError("parse_node cannot be null.")
        return RecentNotebook()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .onenote_source_service import OnenoteSourceService
        from .recent_notebook_links import RecentNotebookLinks

        from .onenote_source_service import OnenoteSourceService
        from .recent_notebook_links import RecentNotebookLinks

        fields: Dict[str, Callable[[Any], None]] = {
            "displayName": lambda n : setattr(self, 'display_name', n.get_str_value()),
            "lastAccessedTime": lambda n : setattr(self, 'last_accessed_time', n.get_datetime_value()),
            "links": lambda n : setattr(self, 'links', n.get_object_value(RecentNotebookLinks)),
            "@odata.type": lambda n : setattr(self, 'odata_type', n.get_str_value()),
            "sourceService": lambda n : setattr(self, 'source_service', n.get_enum_value(OnenoteSourceService)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if not writer:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("displayName", self.display_name)
        writer.write_datetime_value("lastAccessedTime", self.last_accessed_time)
        writer.write_object_value("links", self.links)
        writer.write_str_value("@odata.type", self.odata_type)
        writer.write_enum_value("sourceService", self.source_service)
        writer.write_additional_data_value(self.additional_data)
    

